
default_derive!{
    pub struct NullableObjectRelation {
        rel: Option<String>
    }
}

impl TryFrom<&str> for NullableObjectRelation {
    type Error = &'static str;

    fn try_from(value: &str) -> Result<Self, Self::Error> {
        if value == "null" { //TODO how is null represented in OpenAPI or actual JSON? len()==0?
            Ok(NullableObjectRelation { rel: None })
        }
        else if value.len() <= 255 {
            Ok(NullableObjectRelation { rel: Some(value.to_string()) })
        }
        else {
            Err("Failed to parse NullableObjectRelation")
        }
    }
}

impl Into<String> for NullableObjectRelation {
    fn into(self) -> String {
        match self.rel {
            Some(r) => r,
            None => "null".to_string()
        }
    }
}

#[test]
fn oversized() {
    let rel: Result<NullableObjectRelation, _> = format!("{:>256}", "Test").as_str().try_into();
    assert!(rel.is_err())
}

#[test]
fn valid1() {
    let rel: Result<NullableObjectRelation, _> = "relation".try_into();
    assert_eq!(rel, Ok(NullableObjectRelation { rel: Some("relation".to_string()) }))
}

#[test]
fn valid2() {
    let rel: Result<NullableObjectRelation, _> = "null".try_into();
    assert_eq!(rel, Ok(NullableObjectRelation { rel: None }))
}