use regex_macro::regex;


default_derive!{
    pub struct ObjectId {
        id: String,
    }
}

impl TryFrom<&str> for ObjectId {
    type Error = &'static str;

    fn try_from(value: &str) -> Result<Self, Self::Error> {
        let re = regex!("^[[:digit:]]+$");
        if value.len() <= 10 && re.is_match(value) {
            Ok(ObjectId { id: value.to_string() })
        }
        else {
            Err("Failed to parse ObjectId")
        }
    }
}

impl Into<String> for ObjectId {
    fn into(self) -> String {
        self.id
    }
}

#[test]
fn empty() {
    let id: Result<ObjectId, _> = "".try_into();
    assert!(id.is_err())
}

#[test]
fn oversized() {
    let id: Result<ObjectId, _> = "12345678910".try_into();
    assert!(id.is_err())
}

#[test]
fn valid1() {
    let id: Result<ObjectId, _> = "12345".try_into();
    assert_eq!(id, Ok(ObjectId { id: "12345".to_string() }))
}

#[test]
fn valid2() {
    let id: Result<ObjectId, _> = "0123456789".try_into();
    assert_eq!(id, Ok(ObjectId { id: "0123456789".to_string() }))
}