
default_derive!{
    pub struct ObjectName {
        name: String
    }
}

impl TryFrom<&str> for ObjectName {
    type Error = &'static str;

    fn try_from(value: &str) -> Result<Self, Self::Error> {
        if value.len() <= 131071 {
            Ok(ObjectName { name: value.to_string() })
        }
        else {
            Err("Failed to parse ObjectName")
        }
    }
}

impl Into<String> for ObjectName {
    fn into(self) -> String {
        self.name
    }
}


#[test]
fn oversized() {
    let comment: Result<ObjectName, _> = format!("{:>131072}", "Test").as_str().try_into();
    assert!(comment.is_err())
}

#[test]
fn valid1() {
    let comment: Result<ObjectName, _> = "".try_into();
    assert_eq!(comment, Ok(ObjectName { name: "".to_string() }))
}

#[test]
fn valid2() {
    let comment: Result<ObjectName, _> = "Name".try_into();
    assert_eq!(comment, Ok(ObjectName{name: String::from("Name")}))
}