use regex_macro::regex;


default_derive!{
    pub struct ObjectTemplateVersion {
        version: String,
    }
}

impl TryFrom<&str> for ObjectTemplateVersion {
    type Error = &'static str;

    fn try_from(value: &str) -> Result<Self, Self::Error> {
        let re = regex!("^[[:digit:]]+$");
        if re.is_match(value) {
            Ok(ObjectTemplateVersion { version: value.to_string() })
        }
        else {
            Err("Failed to parse ObjectTemplateVersion")
        }
    }
}

impl Into<String> for ObjectTemplateVersion {
    fn into(self) -> String {
        self.version
    }
}

#[test]
fn empty() {
    let id: Result<ObjectTemplateVersion, _> = "".try_into();
    assert!(id.is_err())
}

#[test]
fn forbidden_char() {
    let id: Result<ObjectTemplateVersion, _> = "123a".try_into();
    assert!(id.is_err())
}

#[test]
fn valid1() {
    let id: Result<ObjectTemplateVersion, _> = "12345".try_into();
    assert_eq!(id, Ok(ObjectTemplateVersion { version: "12345".to_string() }))
}

#[test]
fn valid2() {
    let id: Result<ObjectTemplateVersion, _> = "0123456789".try_into();
    assert_eq!(id, Ok(ObjectTemplateVersion { version: "0123456789".to_string() }))
}